#!/bin/bash

# VirtualBox user
VBOXUSER=myuser
VBOXGRP=vboxusers

#############
# HoneyWall #
#############
HONEYWALL_VM=HoneyWall
# external interface
HONEYWALL_IF=tap2
# internal network
HONEYNET=honeynet
# forwarded ports
HONEYWALL_SSH=10022
HONEYWALL_WALLEYE=10443
# honeypots
HONEYPOT1_VM=Honeypot1
HONEYPOT2_VM=Honeypot2
HONEYPOT3_VM=Honeypot3

#############
# Nephentes #
#############
NEPENTHES_VM=Nepenthes
# external interface
NEPENTHES_IF=tap3
# forwarded ports
NEPENTHES_SSH=20022

#############
# HoneyTrap #
#############
HONEYTRAP_VM=HoneyTrap
HONEYTRAP_IF=tap4
HONEYTRAP_SSH=30022

# darknet-facing physical interface
DARKNET_IF=eth1

# darknet virtual bridge
DARKNET_BRIDGE=br1

# findprog <prog>
function findprog {
	PROG=`which $1`
	if [ ! -x $PROG ]; then
		PROG=`echo $1 | tr [A-Z] [a-z]`
		if [ ! -x $PROG ]; then
			echo "Cannot find $1, exiting."
			exit 1
		fi
	fi
	echo $PROG
}

# mktap <interface>
function mktap {
	sudo tunctl -t $1 -u $VBOXUSER > /dev/null
	sudo ip link set up dev $1
}

# rmtap <interface>
function rmtap {
	sudo ip link set down dev $1
	sudo tunctl -d $1 > /dev/null
}

# mkbridge <bridge> <interface...>
function mkbridge {
	local BRIDGE=$1
	sudo brctl addbr $BRIDGE
	shift
	for IF in $@; do
		sudo brctl addif $BRIDGE $IF
	done
	sudo ip link set up dev $BRIDGE
}

# rmbridge <bridge> <interface...>
function rmbridge {
	local BRIDGE=$1
	shift
	sudo ip link set down dev $BRIDGE
	for IF in $@; do
		sudo brctl delif $BRIDGE $IF
	done
	sudo brctl delbr $BRIDGE
}

# vmsetnic <vm> <nicnc> none
# vmsetnic <vm> <nicnr> <mac> nat
# vmsetnic <vm> <nicnr> <mac> hostif <dev>
# vmsetnic <vm> <nicnr> <mac> intnet <netname>
function vmsetnic {
	local VMSTATUS=`$VBOXMANAGE -nologo showvminfo $1 |grep State:|awk '{ print $2 }'`
	if [ $VMSTATUS == 'saved' ]; then
		echo "VM \"$1\" has a saved state, skipping update to avoid corruption."
	elif [ $VMSTATUS == 'running' ]; then
		echo "VM \"$1\" is currently running, cannot update."
	else
		local ARGS
		local DEFARGS="-cableconnected$2 on -macaddress$2 $3"
		case "$4" in
			none)	ARGS="-nic$2 none";;
			nat)	ARGS="-nic$2 nat $DEFARGS";;
			hostif) ARGS="-nic$2 hostif -hostifdev$2 $5 $DEFARGS";;
			intnet) ARGS="-nic$2 intnet -intnet$2 $5 $DEFARGS";;
		esac
		$VBOXMANAGE -nologo modifyvm $1 $ARGS
	fi
}

# startvm <vm> [headless]
function startvm {
	if [ "$2" == 'headless' ]; then
		$VBOXHEADLESS -startvm $1 > /dev/null &
	else
		$VBOXMANAGE -nologo startvm $1
	fi
}

# stopvm <vm>
function stopvm {
	$VBOXMANAGE -nologo controlvm $1 poweroff
}

function vmresetnatfwd {
	# cleanup old forwarding settings
	local OLDFWD=`$VBOXMANAGE -nologo getextradata $1 enumerate|grep pcnet|awk '{ print $2 }'|sed -e 's/,$//'`
	for KEY in $OLDFWD; do
		$VBOXMANAGE -nologo setextradata $1 $KEY
	done
}

# user manual p. 61
# vmnatfwd <vm> <nicnr> <servicename> <guestport> <hostport>
function vmnatfwd {
	local NIC=$2
	NIC=$[NIC-1]
	$VBOXMANAGE -nologo setextradata $1 "VBoxInternal/Devices/pcnet/$NIC/LUN#0/Config/$3/Protocol" "TCP"
	$VBOXMANAGE -nologo setextradata $1 "VBoxInternal/Devices/pcnet/$NIC/LUN#0/Config/$3/GuestPort" $4
	$VBOXMANAGE -nologo setextradata $1 "VBoxInternal/Devices/pcnet/$NIC/LUN#0/Config/$3/HostPort" $5
}

function setup {
	# allow virtualbox access to tap devices
	if [ `ls -l /dev/net/tun | awk '{ print $4 }'` != $VBOXGRP ]; then
		sudo chown :$VBOXGRP /dev/net/tun
	fi

	# honeywall managment
	vmsetnic $HONEYWALL_VM 1 080027DF6E24 nat
	vmresetnatfwd $HONEYWALL_VM
	vmnatfwd $HONEYWALL_VM 1 roossh 22 $HONEYWALL_SSH
	vmnatfwd $HONEYWALL_VM 1 walleye 443 $HONEYWALL_WALLEYE
	
	# honeywall external
	mktap $HONEYWALL_IF
	vmsetnic $HONEYWALL_VM 2 080027659E52 hostif $HONEYWALL_IF
	
	# honeywall internal
	vmsetnic $HONEYWALL_VM 3 08002716711F intnet $HONEYNET
	
	# honeypot1 internal
	vmsetnic $HONEYPOT1_VM 1 0800272DCEEF intnet $HONEYNET

	# honeypot2 internal
	vmsetnic $HONEYPOT2_VM 1 080027D37916 intnet $HONEYNET

	# honeypot3 internal
	vmsetnic $HONEYPOT3_VM 1 080027DD0907 intnet $HONEYNET
	
	# nephentes managment
	vmsetnic $NEPENTHES_VM 1 080027F54BB0 nat
	vmresetnatfwd $NEPENTHES_VM
	vmnatfwd $NEPENTHES_VM 1 nepssh 20022 $NEPENTHES_SSH
	
	# nephentes external
	mktap $NEPENTHES_IF
	vmsetnic $NEPENTHES_VM 2 0800275DE854 hostif $NEPENTHES_IF

	# nephentes managment
	vmsetnic $HONEYTRAP_VM 1 080027E584AD nat
	vmresetnatfwd $HONEYTRAP_VM
	vmnatfwd $HONEYTRAP_VM 1 honssh 22 $HONEYTRAP_SSH
	
	# nephentes external
	mktap $HONEYTRAP_IF
	vmsetnic $HONEYTRAP_VM 2 080027EC060E hostif $HONEYTRAP_IF
	
	# darknet bridge
	mkbridge $DARKNET_BRIDGE $DARKNET_IF $HONEYWALL_IF $NEPENTHES_IF $HONEYTRAP_IF

	# setup firewall
	sudo iptables -P INPUT ACCEPT
	sudo iptables -F INPUT
	# filter everything but ssh on the management interface
	sudo iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
	sudo iptables -A INPUT -i eth0 -p tcp --dport ! 50022 -j DROP
	sudo iptables -P OUTPUT ACCEPT
	sudo iptables -F OUTPUT
	sudo iptables -P FORWARD DROP
	sudo iptables -F FORWARD
	sudo iptables -A FORWARD -o $DARKNET_BRIDGE -j ACCEPT
	sudo iptables -t nat -F
}

function cleanup {
	# reset firewall
	sudo iptables -P INPUT ACCEPT
	sudo iptables -F INPUT
	sudo iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
	sudo iptables -A INPUT -i eth0 -p tcp --dport ! 50022 -j DROP
	sudo iptables -P OUTPUT ACCEPT
	sudo iptables -F OUTPUT
	sudo iptables -P FORWARD ACCEPT
	sudo iptables -F FORWARD
	sudo iptables -t nat -F
	
	rmbridge $DARKNET_BRIDGE $DARKNET_IF $HONEYWALL_IF $NEPENTHES_IF $HONEYTRAP_IF
	rmtap $HONEYTRAP_IF
	rmtap $NEPENTHES_IF
	rmtap $HONEYWALL_IF
}

function start {
	startvm $HONEYWALL_VM
	startvm $NEPENTHES_VM
	startvm $HONEYTRAP_VM
	startvm $HONEYPOT1_VM
	startvm $HONEYPOT2_VM
	startvm $HONEYPOT3_VM
}

function stop {
	stopvm $HONEYWALL_VM
	stopvm $NEPENTHES_VM
	stopvm $HONEYTRAP_VM
	stopvm $HONEYPOT1_VM
	stopvm $HONEYPOT2_VM
	stopvm $HONEYPOT3_VM
}

VBOXMANAGE=`findprog VBoxManage`
VBOXHEADLESS=`findprog VBoxHeadless`

case "$1" in
	start)
		echo -n "Starting VMs..."
		start
		echo "done."
		;;
	stop)
		echo -n "Stopping VMs..."
		stop
		echo "done."
		;;
	setup)
		sudo -v
		echo -n "Setting up networking..."
		setup
		echo "done."
		;;
	cleanup)
		sudo -v
		echo -n "Bringing down networking..."
		cleanup
		echo "done."
		;;
	*)
		echo "Usage: $0 {start|stop|setup|cleanup}"
		exit 1
		;;
esac

exit 0
